# Copyright (C) 2005 by Aaron Bentley

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
from bzrlib.commands import Command
from bzrlib.errors import MustUseDecorated, BzrOptionError
from bzrlib.option import Option
from bzrlib.workingtree import WorkingTree
import sys
import os
import bzrtools 


class cmd_push(Command):
    """Upload this branch to another location using rsync.

    If no location is specified, the last-used location will be used.  To 
    prevent dirty trees from being uploaded, push will error out if there are 
    unknown files or local changes.  It will also error out if the upstream 
    directory is non-empty and not an earlier version of the branch. 
    """
    _original_command = None
    takes_args = ['location?']
    takes_options = [Option('overwrite', help='Ignore differences between'
                            ' branches and overwrite unconditionally'),
                     Option('no-tree', help='Do not push the working tree,'
                            ' just the .bzr.')]

    def run(self, location=None, overwrite=False, no_tree=False):
        cur_branch = WorkingTree.open_containing(".")[0]
        bzrtools.push(cur_branch, location, overwrite=overwrite, 
                      working_tree=not no_tree)
        
    def run_argv_aliases(self, argv, alias_argv=None):
        """Parse command line and run.
        
        If the command requests it, run the decorated version.
        """
        try:
            super(cmd_push, self).run_argv_aliases(list(argv), alias_argv)
        except (MustUseDecorated, BzrOptionError):
            if self._original_command is None:
                raise
            self._original_command().run_argv_aliases(argv, alias_argv)
